import re
from urllib.request import urlopen
from bs4 import BeautifulSoup


def get_id_number(author_id):
    pattern = re.compile("([^.-]+)")
    return pattern.search(author_id).group()


def get_author_description(soup, id_number):
    cell = soup.find("span", {"id": "freeTextauthor" + id_number})
    if cell:
        return cell.text.strip()
    return None


def get_author_image(soup, author_name):
    cell = soup.find("img", {"alt": author_name, "itemprop": "image"})
    if cell:
        return cell.attrs.get("src")
    return None


def scrape_author(author_id):
    url = "https://www.goodreads.com/author/show/" + author_id
    source = urlopen(url)
    soup = BeautifulSoup(source, "html.parser")

    author_name = soup.find("span", {"itemprop": "name"}).text.strip()
    id_number = get_id_number(author_id)

    return {
        "author_id_title": author_id,
        "author_id": id_number,
        "author_name": author_name,
        "author_url": url,
        "author_image": get_author_image(soup, author_name),
        "author_description": get_author_description(soup, id_number),
    }



from argparse import Namespace
import json
from urllib.request import urlopen
from bs4 import BeautifulSoup
import re


def get_user_name(soup: BeautifulSoup):
    return soup.find(id="profileNameTopHeading").text.strip()


def get_num_ratings(soup: BeautifulSoup):
    container = soup.find("div", attrs={"class": "profilePageUserStatsInfo"})
    return int(re.findall(r"\d+", container.find("a").text)[0])


def get_avg_rating(soup: BeautifulSoup):
    container = soup.find("div", attrs={"class": "profilePageUserStatsInfo"})
    return float(re.findall(r"\d*\.?\d+", container.find_all("a")[1].text)[0])


def get_num_reviews(soup: BeautifulSoup):
    container = soup.find("div", attrs={"class": "profilePageUserStatsInfo"})
    return int(re.findall(r"\d+", container.find_all("a")[2].text)[0])


def get_user_info(args: Namespace):
    if args.skip_user_info:
        return

    print("Scraping user...")

    user_id: str = args.user_id
    output_file: str = args.output_dir + "user.json"
    url = "https://www.goodreads.com/user/show/" + user_id
    source = urlopen(url)
    soup = BeautifulSoup(source, "html.parser")

    data = {
        "user_id": user_id,
        "user_name": get_user_name(soup),
        "num_ratings": get_num_ratings(soup),
        "average_rating": get_avg_rating(soup),
        "num_reviews": get_num_reviews(soup),
    }

    file = open(output_file, "w")
    json.dump(data, file, indent=2)
    file.close()

    print("👤 Scraped user")

    if not args.skip_shelves:
        print()



from argparse import Namespace
import json
from urllib.request import urlopen
import os
from bs4 import BeautifulSoup
import re

from scraper import books


RATING_STARS_DICT = {
    "it was amazing": 5,
    "really liked it": 4,
    "liked it": 3,
    "it was ok": 2,
    "did not like it": 1,
    "": None,
}


def get_shelf_url(user_id, shelf, page):
    url = (
        "https://www.goodreads.com/review/list/"
        + user_id
        + "?shelf="
        + shelf
        + "&page="
        + str(page)
        + "&print=true"
    )
    source = urlopen(url)
    return BeautifulSoup(source, "html.parser")


def get_id(book_row):
    cell = book_row.find("td", {"class": "field title"})
    title_href = cell.find("div", {"class": "value"}).find("a")
    return title_href.attrs.get("href").split("/")[-1]


def get_rating(book_row):
    cell = book_row.find("td", {"class": "field rating"})
    str_rating = cell.find("div", {"class": "value"}).find("span").attrs.get("title")
    return RATING_STARS_DICT.get(str_rating)


def get_dates_read(book_row):
    cell = book_row.find("td", {"class": "field date_read"})
    dates = cell.find("div", {"class": "value"}).findChildren(
        "div", {"class": "date_row"}
    )
    date_arr = []
    for date in dates:
        date_text = date.text.strip()
        if date_text != "not set":
            date_arr += [date_text]
    return date_arr


def get_shelf(args: Namespace, shelf: str):
    print("Scraping '" + shelf + "' shelf...")
    user_id: str = args.user_id
    output_dir: str = args.output_dir + "books/"
    page = 1

    while True:
        soup = get_shelf_url(user_id, shelf, page)

        no_content = soup.find("div", {"class": "greyText nocontent stacked"})
        if no_content:
            break

        books_table = soup.find("tbody", {"id": "booksBody"})
        book_rows = books_table.findChildren("tr", recursive=False)

        # Loop through all books in the page
        for book_row in book_rows:
            book_id = get_id(book_row)
            file_path = output_dir + book_id + ".json"

            book = None
            changed = False

            # If the book has already been scraped, just add the shelf
            if os.path.exists(file_path):
                file = open(file_path, "r")
                book = json.load(file)
                if shelf not in book["shelves"]:
                    book["shelves"].append(shelf)
                    print("✅ Updated " + book_id)
                    changed = True
                file.close()
            # If not already scraped, scrape the book and add the shelf
            else:
                book = books.scrape_book(book_id, args)
                book["rating"] = get_rating(book_row)
                book["dates_read"] = get_dates_read(book_row)
                book["shelves"] = [shelf]
                print("🎉 Scraped " + book_id)
                changed = True

            if changed:
                # Write the json file for the book
                file = open(file_path, "w")
                json.dump(book, file, indent=2)
                file.close()

        page += 1

    print()


def get_all_shelves(args: Namespace):
    if args.skip_shelves:
        return

    user_id: str = args.user_id
    output_dir: str = args.output_dir + "books/"
    url = "https://www.goodreads.com/user/show/" + user_id
    source = urlopen(url)
    soup = BeautifulSoup(source, "html.parser")

    os.makedirs(output_dir, exist_ok=True)

    shelves_div = soup.find("div", {"id": "shelves"})
    shelf_links = shelves_div.findChildren("a")

    for link in shelf_links:
        base_url = link.attrs.get("href")
        shelf: str = re.search(r"\?shelf=([^&]+)", base_url).group(1)
        get_shelf(args, shelf)



"""
Source: https://github.com/maria-antoniak/goodreads-scraper/blob/master/get_books.py
"""
import re
from urllib.request import urlopen
from bs4 import BeautifulSoup
from argparse import Namespace

from scraper import author


# def get_rating_distribution(soup: BeautifulSoup):
#     distribution = re.findall(r"renderRatingGraph\([\s]*\[[0-9,\s]+", str(soup))[0]
#     distribution = " ".join(distribution.split())
#     distribution = [int(c.strip()) for c in distribution.split("[")[1].split(",")]
#     distribution_dict = {
#         5: distribution[0],
#         4: distribution[1],
#         3: distribution[2],
#         2: distribution[3],
#         1: distribution[4],
#     }
#     return distribution_dict


def get_author_id(soup: BeautifulSoup):
    author = soup.find("div", {"class": "ContributorLinksList"}).find("a")
    author_url = author.attrs.get("href")
    return author_url.split("/")[-1]


def get_genres(soup: BeautifulSoup):
    genres = []
    genres_container = soup.find("div", {"data-testid": "genresList"})

    if genres_container:
        genre_links = genres_container.find("ul").find("span").find_all("a")

        for link in genre_links:
            genre = link.find("span").text
            genres.append(genre)
        return genres

    return None


def get_average_rating(soup: BeautifulSoup):
    average_rating = soup.find(
        "div", {"class": "RatingStatistics__rating"}
    ).text.strip()

    if average_rating:
        return float(average_rating)
    return ""


def get_num_reviews(soup: BeautifulSoup):
    num_reviews = soup.find("span", {"data-testid": "reviewsCount"})

    if num_reviews:
        num_reviews = re.search(
            r"(\d{1,3}(,\d{3})*(\.\d+)?)", num_reviews.text.strip()
        ).group(1)
        return int(num_reviews.replace(",", ""))
    return ""


def get_num_ratings(soup: BeautifulSoup):
    num_ratings = soup.find("span", {"data-testid": "ratingsCount"})

    if num_ratings:
        num_ratings = re.search(
            r"(\d{1,3}(,\d{3})*(\.\d+)?)", num_ratings.text.strip()
        ).group(1)
        return int(num_ratings.replace(",", ""))
    return ""


def get_num_pages(soup: BeautifulSoup):
    num_pages_container = soup.find("p", {"data-testid": "pagesFormat"})
    num_pages = re.search(
        r"(\d{1,3}(,\d{3})*(\.\d+)?)", num_pages_container.text.strip()
    )

    if num_pages:
        num_pages = num_pages.group(1)
        return int(num_pages.replace(",", ""))
    return None


def get_year_first_published(soup: BeautifulSoup):
    year_first_published = soup.find("p", {"data-testid": "publicationInfo"})
    if year_first_published:
        year_first_published = year_first_published.string
        return re.search(r"([0-9]{3,4})", year_first_published).group(1)
    else:
        return None


def get_series_uri(soup: BeautifulSoup):
    title_section = soup.find("h1", {"data-testid": "bookTitle"}).parent
    series_container = title_section.find("h3")

    if series_container:
        return series_container.find("a").get("href")
    else:
        return None


# def get_series_name(soup: BeautifulSoup):
#     title_section = soup.find("h1", {"data-testid": "bookTitle"}).parent
#     series_container = title_section.find("h3")

#     if series_container:
#         series_name = re.search(r"\((.*?)\)", series_container.find("a").text).group(1)
#         return series_name
#     else:
#         return None


def get_image(soup: BeautifulSoup):
    return soup.find("img", {"class": "ResponsiveImage"}).attrs.get("src")


def get_description(soup: BeautifulSoup):
    description_div = soup.find("div", {"data-testid": "description"}).find("span")

    if description_div:
        return description_div.text
    else:
        return None


def get_title(soup: BeautifulSoup):
    return " ".join(soup.find("h1", {"data-testid": "bookTitle"}).text.split())


def get_id(book_id: str):
    pattern = re.compile("([^.-]+)")
    return pattern.search(book_id).group()


def scrape_book(book_id: str, args: Namespace):
    url = "https://www.goodreads.com/book/show/" + book_id
    source = urlopen(url)
    soup = BeautifulSoup(source, "html.parser")

    book = {
        "book_id_title": book_id,
        "book_id": get_id(book_id),
        "book_title": get_title(soup),
        "book_description": get_description(soup),
        "book_url": url,
        "book_image": get_image(soup),
        # "book_series": get_series_name(soup),
        "book_series_uri": get_series_uri(soup),
        "year_first_published": get_year_first_published(soup),
        "num_pages": get_num_pages(soup),
        "genres": get_genres(soup),
        "num_ratings": get_num_ratings(soup),
        "num_reviews": get_num_reviews(soup),
        "average_rating": get_average_rating(soup),
        # "rating_distribution": get_rating_distribution(soup),
    }

    if not args.skip_authors:
        book["author"] = author.scrape_author(get_author_id(soup))

    return book



import argparse
import os

from scraper import shelves
from scraper import user


def scrape_user(args: argparse.Namespace):
    user.get_user_info(args)
    shelves.get_all_shelves(args)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--user_id", type=str, required=True)
    parser.add_argument("--output_dir", type=str, default="goodreads-data")
    parser.add_argument("--skip_user_info", type=bool, default=False)
    parser.add_argument("--skip_shelves", type=bool, default=False)
    parser.add_argument("--skip_authors", type=bool, default=False)

    args = parser.parse_args()

    args.output_dir = (
        args.output_dir if args.output_dir.endswith("/") else args.output_dir + "/"
    )

    os.makedirs(args.output_dir, exist_ok=True)
    scrape_user(args)


if __name__ == "__main__":
    main()
